//ROS Header
#include <ros/ros.h>
#include <sensor_msgs/JointState.h>
#include <tf/transform_broadcaster.h>
#include <nav_msgs/Odometry.h>
#include "std_msgs/String.h"
#include "std_msgs/Int8.h"
#include "std_msgs/Int16.h"
#include "std_msgs/Float32.h"
#include "std_msgs/Float64.h"
#include "ihm.h"

//----------------ENRIC----------------
#include <sensor_msgs/Joy.h> 
#include <iomanip>
#include <std_msgs/Bool.h>
#include <algorithm> //for computing the minimum in test terrain

//--------------END ENRIC---------------

//Socket Header
#include "SockBase.h"
#include <errno.h>
#include "TCPSock.h"
#include <sstream>
#include <cstdlib>
#include <cstring>
#include <string>
#include <iostream>
#include <stdio.h> 
#include <math.h>
#include <cmath>
#include <bitset> 

#include <algorithm>

using namespace std;

//------------------------ ROS Template ---------------------------
double width_robot = 0.1;
double vl = 0.0;
double vr = 0.0;

ros::Time last_time;

double right_enc = 0.0;
double left_enc = 0.0;
double right_enc_old = 0.0;
double left_enc_old = 0.0;
double distance_left = 0.0;
double distance_right = 0.0;
double ticks_per_meter = 100;
double x = 0.0;
double y = 0.0;
double th = 0.0;
geometry_msgs::Quaternion odom_quat;
//----------------------- END Template ---------------------------

//----------------------- PERE Code ------------------------------
//Global variable declaration
int errno;

char * addrRobot;
char * addrPIC;
char * addrPc;
char * addrIMU;

int PC_send_period=1000;	//Must be a multiple of 5ms
int rcv_count=0;			//Counter for number of packages recieved from the dsPIC (5ms counter)
int send_PIC=0;				//Flag to start transmision to the dsPIC
int sendTimer=0;			//Flag per controlar que el thread de sendPIC no envii massa rapid - Flag to control the thread to send PIC not send too fast
unsigned int real_time=0;	//Acumulated robot time
int nBytesInt = 0; 			//Variable per controlar els bytes a enviar cap al PIC

int PC_send_bytes=0;		//Number of bytes to be send to the PC
int Buf_PC_send_bytes[5];
int indexRebre = 0;
unsigned char bufModeCom[3];

//Define configuration for socket PC
static int servSock=-1;                 	/* Socket descriptor for server */
static int clntSock=-1;                 	/* Socket descriptor for client */
static struct sockaddr_in echoServAddr; 	/* Local address */
static struct sockaddr_in echoClntAddr; 	/* Client address */
static unsigned int clntLen;            	/* Length of client address data structure */
static unsigned short echoServPort=15000;   //Server port
#define MAXPENDING 	5

int sockPIC;
int sockIMU;
int start_send=0;

//Load Threads
void * sendPIC(void *num);
void * recvPIC(void *num);

int composa_PC(int index);
void startSocketPIC(void);
void stopSocketPIC(void);
void gestio_int (int numero);
int calculaPaquet_PIC(void);

//Variables used to convert values ​​bufUp's a format that can be published
//This is because the ROS membranes not publish unsigned char *
float Vtemsp = 0;
float Vcodi_err = 0;

int VVel_FL= 0;
int VVel_FR= 0;
int VVel_BL= 0;
int VVel_BR= 0;
int VInt_FL = 0;
int VInt_FR = 0;
int VInt_BL = 0;
int VInt_BR = 0;

char VBat = 0;
char VVcc = 0;

int Scont_FL= 0;
int Scont_FR= 0;
int Scont_BL= 0;
int Scont_BR= 0;

int VPos_X = 0;
int VPos_Y = 0;
int VTheta =0;

double dVVel_FL= 0;
double dVVel_FR= 0;
double dVVel_BL= 0;
double dVVel_BR= 0;
double dVInt_FL = 0;
double dVInt_FR = 0;
double dVInt_BL = 0;
double dVInt_BR = 0;

double dVBat = 0;

double dVVcc = 0;

int dScont_FL= 0;
int dScont_FR= 0;
int dScont_BL= 0;
int dScont_BR= 0;

double dPos_X = 0;
double dPos_Y = 0;
double dTheta = 0;

double dEnc_L= 0;
double dEnc_R= 0;

double VEnc_L= 0;
double VEnc_R= 0;


//----------------------- END PERE Code ------------------------------



//------------------------ ENRIC Code --------------------------------

int testSpeed = 1;

int checkTerrain = 0;		//by default check terrain will be 0. This will be a flag that will indicate 
							//when should we check if there is a new terrain
						
int PID = 1;
bool flag_wrong_test;

int lastMovement = 1;		//This variable will store the last type of movement done by the robot. This 
							//can be forward, full turning, turning doing arcs. It will be used to reset
							//the PID whenever a change of type of movement is done
							
float terrtype = 1;
int terrainL = 0;
int terrainR = 0;
int terreny = 0;
							
bool flagTestRepeated = false;

void resetPID(){
	/* This function will be used in order to reset the PID of the 4 wheels
	 * This is done because the wheels tend to finish their movement. However
	 * they cannot drift totally in some terrains causing residual intensities.
	 * This can affect negatively in the readings for computing the power.
	 */
	
		ros::Time start_time = ros::Time::now();
		ros::Duration timeout(0.01);					//duration of the experiment in seconds
		cout << "Reset PID" << endl;
		while(ros::Time::now() - start_time < timeout){
			
			bufDown[0]= 0x40;			//New parameters for the PID
			bufDown[20]= 0x00; 			//Kp - High
			bufDown[21]= 0x00; 			//Kp - Low
			bufDown[22]= 0x00;			//Ki - High
			bufDown[23]= 0x00;			//Ki - Low
			
			send_PIC=1;					//send to the PIC activated
			ros::Duration(0.01).sleep();	//sleep the procces for 0.1 seconds
		}
		
		bufDown[0]= 0x40;				//New parameters for the PID
		bufDown[20]= 0x00; 				//Kp - High
		bufDown[21]= 0x03; 				//Kp - Low
		bufDown[22]= 0x01;				//Ki - High
		bufDown[23]= 0xF4;				//Ki - Low
		send_PIC = 1;
		
		ros::Duration(0.01).sleep();		//sleep during half a second so we are sure that the instructions
										//have been received by the dsPIC
		bufDown[0]= 0x00;				//there will be no new instructions for the PID
	
}

void runTerrainTest(){
	
	//intensities will be stores in these two variables that will be computing
	//the mean of the two front wheels. With these values it will be possible
	//to map to map a function that will tell in which type of floor the robot
	//is placed
	int L_int_b = 0;	//left intensity buffer
	int R_int_b = 0;	//right intensity buffer
	int ind = 0;
	int mean_current = 0;
	// Inicialitzacio variables dist
	int dist1L;
	int	dist2L;
	int	dist3L;
	int dist4L;
	int dist5L;
	int dist6L;
	int dist7L;
	int dist9L;
	int	dist1R;
	int	dist2R;
	int	dist3R;
	int dist4R;
	int	dist5R;
	int	dist6R;
	int	dist7R;
	int dist9R;
	// Inicialització variables de terreny
	float a1L;
	float b1L;
	float c1L;
	float a1R;
	float b1R;
	float c1R;
	float a2L;
	float b2L;
	float c2L;
	float a2R;
	float b2R;
	float c2R;
	float a3L;
	float b3L;
	float c3L;
	float a3R;
	float b3R;
	float c3R;
	float a4L;
	float b4L;
	float c4L;
	float a4R;
	float b4R;
	float c4R;
	float a5L;
	float b5L;
	float c5L;
	float a5R;
	float b5R;
	float c5R:
	float a6L;
	float b6L;
	float c6L;
	float a6R;
	float b6R;
	float c6R;
	float a7L;
	float b7L;
	float c7L;
	float a7R;
	float b7R;
	float c7R;
	float a9L;
	float b9L;
	float c9L;
	float a9R;
	float b9R;
	float c9R;
	
	//int terrainL;
	//int terrainR;
	

	cout << "Change of terrain detected!" << endl;
		
/* --------------------------- PART 1 ----------------------------------
 * 
 * In this first part the PID is set to 0 so there is no intensities on
 * the wheels causing bad readings 
 * 
 * -------------------------------------------------------------------*/
		ros::Time start_time = ros::Time::now();
		ros::Duration timeout(1);					//duration of the experiment in seconds
		cout << "Setting the motors properly" << endl;
		while(ros::Time::now() - start_time < timeout){
			
			bufDown[0]= 0x40;			//New parameters for the PID
			bufDown[20]= 0x00; 			//Kp - High
			bufDown[21]= 0x00; 			//Kp - Low
			bufDown[22]= 0x00;			//Ki - High
			bufDown[23]= 0x00;			//Ki - Low
			
			send_PIC=1;					//send to the PIC activated
			ros::Duration(0.1).sleep();	//sleep the procces for 0.1 seconds
		}
		
		bufDown[0]= 0x40;				//New parameters for the PID
		bufDown[20]= 0x00; 				//Kp - High
		bufDown[21]= 0x03; 				//Kp - Low
		bufDown[22]= 0x01;				//Ki - High
		bufDown[23]= 0xF4;				//Ki - Low
		send_PIC = 1;
		
		ros::Duration(0.5).sleep();		//sleep during half a second so we are sure that the instructions
										//have been received by the dsPIC
		bufDown[0]= 0x00;				//there will be no new instructions for the PID
		
		
/* --------------------------- PART 2 ----------------------------------
 * 
 * During this step the robot will be going forward during some seconds
 * (set on 'timeout2'). The values of the intensities of the two front
 * wheels will be stored. This part is splited in two different parts
 * since it is needed a stabilization of the robot in order to start 
 * getting correct data
 *
 *--------------------------------------------------------------------*/	
		
		ros::Time start_time4 = ros::Time::now();
		ros::Duration timeout4(2.0);					//duration of the stabilization in seconds
		
		cout << "Running terrain experiment for " << timeout4.toSec() << " seconds" << endl;
		
		while(ros::Time::now() - start_time4 < timeout4){
			
			bufDown[4] = 0x28;	//Front Left wheel
			bufDown[5] = 0x28;	//Front Right wheel
			bufDown[6] = 0x20;	//Rear Left wheel
			bufDown[7] = 0x20;	//Rear Right wheel

			send_PIC=1;					//send to the PIC activated
			ros::Duration(0.1).sleep();	//sleep the procces for 0.1 seconds
			
		}
		
		
		ros::Time start_time2 = ros::Time::now();
		ros::Duration timeout2(1.0);					//duration of the experiment in seconds
														//here the data will be stored
		
		cout << "Running terrain experiment for " << timeout2.toSec() << " seconds" << endl;
		
		while(ros::Time::now() - start_time2 < timeout2){
			
			bufDown[4] = 0x28;	//Front Left wheel
			bufDown[5] = 0x28;	//Front Right wheel
			bufDown[6] = 0x20;	//Rear Left wheel
			bufDown[7] = 0x20;	//Rear Right wheel
			
			cout << "L: " << (dVInt_FL/1000.0)*dVBat*Scont_FL << " " << (dVInt_FL/1000.0) << " " << Scont_FL << " R: " << (dVInt_FR/1000.0)*dVBat*Scont_FR << " " << (dVInt_FR/1000.0) << " " << Scont_FR << endl;
			
			L_int_b = L_int_b + (dVInt_FL/1000.0)*dVBat*Scont_FL;
			R_int_b = R_int_b + (dVInt_FR/1000.0)*dVBat*Scont_FR;
			ind++;
			
			send_PIC=1;					//send to the PIC activated
			ros::Duration(0.1).sleep();	//sleep the procces for 0.1 seconds
			
		}

/*---------------------------- PART 3 ----------------------------------
* 
* Stop the robot and evaluation of the results obtained during the test
* 
* --------------------------------------------------------------------*/
		 
		cout << "Experiment done" << endl;
		bufDown[4] = 0x00;	//Front Left wheel
		bufDown[5] = 0x00;	//Front Right wheel
		bufDown[6] = 0x00;	//Rear Left wheel
		bufDown[7] = 0x00;	//Rear Right wheel
		send_PIC=1;	
		
		L_int_b = L_int_b / ind;	//computing the mean of the two powers
		R_int_b = R_int_b / ind;
		
		cout << "Mean power of FL motor: " << L_int_b << endl;
		cout << "Mean power of FR motor: " << R_int_b << endl;
		
		
		//Dadres terreny
		
		a1L = 21.0568;
		b1L = -1;
		c1L = 439.2821;
		a1R = 24.3999;
		b1R = -1;
		c1R = 418.3002;
		
		
		
		//Compacted Gravel (exterior al pati)
		/*a2L = 32.7185;
		b2L = -1;
		c2L = 1577.3;
		a2R = 27.9537;
		b2R = -1;
		c2R = 1920.5;*/
		a2L = 35.4053;
		b2L = -1;
		c2L = 1443.0;
		a2R = 18.3099;
		b2R = -1;
		c2R = 2124.4;
		
		//Sand
		/*a3L = 15.7314;
		b3L = -1;
		c3L = 547.0568;
		a3R = 14.7892;
		b3R = -1;
		c3R = 385.2138;*/
		a3L = 15.5443;
		b3L = -1;
		c3L = 424.7992;
		a3R = 18.6661;
		b3R = -1;
		c3R = 318.5861;
		
		//Grass
		/*a4L = 35.6003;
		b4L = -1;
		c4L = 1194;
		a4R = 37.779;
		b4R = -1;
		c4R = 993.6881;*/
		a4L = 36.9373;
		b4L = -1;
		c4L = 968.7114;
		a4R = 36.0161;
		b4R = -1;
		c4R = 832.4466;
		
		//Mosaic P2
		a5L = 14.1223;
		b5L = -1;
		c5L = 320.1320;
		a5R = 12.0470;
		b5R = -1;
		c5R = 475.5917;
		
		//Asfalt
		a6L = 14.1223;
		b6L = -1;
		c6L = 320.1320;
		a6R = 12.0470;
		b6R = -1;
		c6R = 475.5917;
		
		//Ciment barracons
		a7L = 14.1223;
		b7L = -1;
		c7L = 320.1320;
		a7R = 12.0470;
		b7R = -1;
		c7R = 475.5917;
		
		//Pedra 
		a9L = 14.1223;
		b9L = -1;
		c9L = 320.1320;
		a9R = 12.0470;
		b9R = -1;
		c9R = 475.5917;
		
		int speed = 40;			//speed of the test in decimal
		
		// Visualització dades de terreny
		
		dist1L = abs(a1L*speed+c1L-L_int_b);
		
		dist2L = abs(a2L*speed+c2L-L_int_b);
		
		dist3L = abs(a3L*speed+c3L-L_int_b);
		
		dist4L = abs(a4L*speed+c4L-L_int_b);
		
		dist5L = abs(a5L*speed+c5L-L_int_b);
		
		dist6L = abs(a6L*speed+c6L-L_int_b);
		
		dist7L = abs(a7L*speed+c7L-L_int_b);
		
		dist9L = abs(a9L*speed+c9L-L_int_b);
		
		dist1R = abs(a1R*speed+c1R-R_int_b);
		
		dist2R = abs(a2R*speed+c2R-R_int_b);
		
		dist3R = abs(a3R*speed+c3R-R_int_b);
		
		dist4R = abs(a4R*speed+c4R-R_int_b);
		
		dist5R = abs(a5R*speed+c5R-R_int_b);
		
		dist6R = abs(a6R*speed+c6R-R_int_b);
		
		dist7R = abs(a7R*speed+c7R-R_int_b);
		
		dist9R = abs(a9R*speed+c9R-R_int_b);

		cout << "Dist ceramic  Left: " << dist1L << endl;
		cout << "Dist outside  Left: " << dist2L << endl;
		cout << "Dist sand     Left: " << dist3L << endl;
		cout << "Dist grass    Left: " << dist4L << endl;
		cout << "Dist Mosaic P2  Left: " << dist5L << endl;
		cout << "Dist Asfalt   Left: " << dist6L << endl;
		cout << "Dist Ciment barr  Left: " << dist7L << endl;
		cout << "Dist pedra    Left: " << dist9L << endl;
		cout << "Dist ceramic Right: " << dist1R << endl;
		cout << "Dist outside Right: " << dist2R << endl;
		cout << "Dist sand    Right: " << dist3R << endl;
		cout << "Dist grass   Right: " << dist4R << endl;
		cout << "Dist Mosaic P2   Right: " << dist5R << endl;
		cout << "Dist Asfalt  Right: " << dist6R << endl;
		cout << "Dist Ciment barr Right: " << dist7R << endl;
		cout << "Dist pedra   Right: " << dist9R << endl;
		/*if (dist3L < dist1L){
			terrainL = 3;		//sorra
		}else if (dist2L < dist1L){
			terrainL = 2;		//grava
		}else{
			terrainL = 1;		//interior
		}
		
		if (dist3R < dist1R){
			terrainR = 3;		//sorra
		}else if (dist2R < dist1R){
			terrainR = 2;		//grava
		}else{
			terrainR = 1;		//interior
		}*/

		terrainL = min(dist9L, min(dist7L, min(dist6L, min(dist5L, min(dist1L, min(dist2L, min(dist3L, dist4L)))))));
		terrainR = min(dist9R, min(dist7R, min(dist6R, min(dist5R, min(dist1R, min(dist2R, min(dist3R, dist4R)))))));
		
		if(terrainL==dist1L){
			terrainL = 1;
		}else if(terrainL==dist2L){
			terrainL = 2;
		}else if(terrainL==dist3L){
			terrainL = 3;
		}else if(terrainL==dist4L){
			terrainL = 4;
		}else if(terrainL==dist5L){
			terrainL = 5;
		}else if(terrainL==dist6L){
			terrainL = 6;
		}else if(terrainL==dist6L){
			terrainL = 7;
		}else if(terrainL==dist9L){
			terrainL = 9;
		}
		
		
		if(terrainR==dist1R){
			terrainR = 1;
		}else if(terrainR==dist2R){
			terrainR = 2;
		}else if(terrainR==dist3R){
			terrainR = 3;
		}else if(terrainR==dist4R){
			terrainR = 4;
		}else if(terrainR==dist5R){
			terrainR = 5;
		}else if(terrainR==dist6R){
			terrainR = 6;
		}else if(terrainR==dist6R){
			terrainR = 7;
		}else if(terrainR==dist9R){
			terrainR = 9;
		}
		
		cout << endl;
		cout << "Terrain detected by left wheel:  " << terrainL << endl;
		cout << "Terrain detected by right wheel: " << terrainR << endl;
		cout << endl;
		 // Visulització missatge de terreny
		 
		if (terrainL == terrainR){
			
			flag_wrong_test = false;
			
			switch(terrainL){
				
				case 1:
						cout << endl << "Terrain is ceramic tile" << endl << endl;
						terrtype = 1;
						break;
				case 2:
						cout << endl << "Terrain is compacted gravel" << endl << endl;
						terrtype = 2;
						break;
				case 3:
						cout << endl << "Terrain is sand" << endl << endl;
						terrtype = 3;
						break;	
				case 4:
						cout << endl << "Terrain is grass" << endl << endl;
						terrtype = 4;
						break;
				case 5:
						cout << endl << "Terrain is Mosaic P2" << endl << endl;
						terrtype = 5;
						break;
				case 6:
						cout << endl << "Terrain is Asfalt" << endl << endl;
						terrtype = 6;
						break;
				case 7:
						cout << endl << "Terrain is Asfalt" << endl << endl;
						terrtype = 7;
						break;
				case 9:
						cout << endl << "Terrain is Pedra barracons" << endl << endl;
						terrtype = 9;
						break;		
			}
		}else{
			cout << "An error ocurred during the experiment and it should be repeated" << endl;
			flag_wrong_test = true;
		}
		
		
/* --------------------------- PART 4 ----------------------------------
 * 
 * Relaxation step. Because the wheels are not spinning at the same
 * speed and the PID will try to finish the movement of the 4 wheels the
 * robot will not be in a relaxed position when it ends the test. Taht's
 * why it is needed to relax the robot setting again the parameters of
 * the PID to 0
 * 
 * -------------------------------------------------------------------*/
		ros::Time start_time3 = ros::Time::now();
		ros::Duration timeout3(0.5);					//duration of the experiment in seconds
		cout << "Setting the motors properly" << endl;
		while(ros::Time::now() - start_time3 < timeout3){
			
			bufDown[0]= 0x40;
			bufDown[20]= 0x00; 
			bufDown[21]= 0x00; 
			bufDown[22]= 0x00;
			bufDown[23]= 0x00;
			
			bufDown[4] = 0x00;	//Front Left wheel
			bufDown[5] = 0x00;	//Front Right wheel
			bufDown[6] = 0x00;	//Rear Left wheel
			bufDown[7] = 0x00;	//Rear Right wheel
			
			send_PIC=1;					//send to the PIC activated
			ros::Duration(0.1).sleep();	//sleep the procces for 0.1 seconds
		}

		bufDown[0]= 0x40;
		bufDown[20]= 0x00; 
		bufDown[21]= 0x03; 
		bufDown[22]= 0x01;
		bufDown[23]= 0xF4;
		send_PIC = 1;
		
		ros::Duration(0.5).sleep();
		
		bufDown[0]= 0x00;
	
}
/*
void runTest_Callback(const std_msgs::Bool::ConstPtr& runTest){
/* If the runTest flag is activated the program goes to runTerrainTest
 * which will be performed in order to identify in which kind of terrain
 * is the robot
  	
 
	if (flagTestRepeated == false){
		cout << "Calling runTerrainTest" << endl;
		runTerrainTest();
		flagTestRepeated = true;
	}
 
	

}*/

void joyCallback (const sensor_msgs::Joy::ConstPtr& joy_data){
/* This function responds to any change on the state of the joystick. It
 * is mainly used to detect when the buttons are pressed and act according
 * the one that has been pressed
 */
	if (joy_data->buttons[7] == 1){	//this button will restart the PID
		resetPID();
	}
	
	if (joy_data->buttons[2] == 1){	//by now the activation is done by pressing a button on the joystick
		runTerrainTest();
		terreny = 1;
		
	}	
}
				
//----------------------- END ENRIC Code -----------------------------

void cmd_velCallback(const geometry_msgs::Twist &twist_aux)
{
/**********************************************
void cmd_velCallback is to listen from cmd_vel topic and send its data to the robot
* Linear in meter per second
* Angular in radian per second
***********************************************/

	geometry_msgs::Twist twist = twist_aux;
	double vel_x = twist_aux.linear.x;
		   vel_x = vel_x * 100; //convert from meter to centimeter, because robot only receive in centimeter per second unit metric
		   //vel_x = vel_x / 0.84; //this is ratio of wheel diameter
		   vel_x = vel_x; //WITHOUT diameter
		   //vel_x = vel_x + 0.5; //Ceil function is broken here, so I add 0.5 to make ceil function
	double vel_th = twist_aux.angular.z;
	if(vel_th != 0)
		{
			if(vel_th > 0)
			{
				vel_th = (vel_th * 33.35) + 8.67; //convert from meter to radian, because robot only receive in centimeter per second unit metric
				vel_th = vel_th / 0.84;//this is ratio of wheel diameter
			}
			if(vel_th < 0)
			{
				vel_th = (vel_th * 33.35) - 8.67; //convert from meter to radian, because robot only receive in centimeter per second unit metric
				vel_th = vel_th / 0.84;//this is ratio of wheel diameter
			}
			
		}
		   
		   //vel_th = vel_th + 0.5; //Ceil function is broken here, so I add 0.5 to make ceil function
	//double right_vel = 0.0;		useles code?
	//double left_vel = 0.0;		useles code?

	bufDown[0]= 0x00; // 0000 0000
	bufDown[1]= 0xFF; // 1111 1111
	bufDown[2]= 0x00; // 0000 0111
	bufDown[3]= 0x03; // 0000 0011
	
	unsigned char val_speed_x = vel_x;
	unsigned char val_speed_th = vel_th;
	unsigned char val_speed_inv_th = -(vel_th);

	
	if(abs(vel_x) < 20 ){
		// turning
		//right_vel = vel_th * width_robot / 2.0; 	useles code?
		//left_vel = (-1) * right_vel;				useles code?
		

		bufDown[4]= val_speed_th;
		bufDown[5]= val_speed_inv_th ;
		bufDown[6]= val_speed_th;
		bufDown[7]= val_speed_inv_th ;
		
		if (lastMovement==2 || lastMovement==3){
			resetPID();
			lastMovement=1;
			//cout << "Start turning" << endl;
		}
		
	}
	else if(abs(vel_th) < 20){
		// forward / backward
		//left_vel = right_vel = vel_x; useles code?
		
		if (lastMovement==1 || lastMovement==3){
			resetPID();
			lastMovement=2;
			//cout << "Go forward" << endl;
			flagTestRepeated = false;
		}
		
		
		
		bufDown[4]= val_speed_x;
		bufDown[5]= val_speed_x;
		bufDown[6]= val_speed_x;
		bufDown[7]= val_speed_x;
		
	}
	else{
		// moving doing arcs
		//left_vel = vel_x - vel_th * width_robot / 2.0; 	useles code?
		//right_vel = vel_x + vel_th * width_robot / 2.0; 	useles code?
		
		if (lastMovement==1 || lastMovement==2){
			resetPID();
			lastMovement=3;
			//cout << "Doing arcs" << endl;
		}
		
		if(vel_th >= 0 and vel_x >=0 )
		{
			bufDown[4]= 0x28;//val_speed_th ;
			bufDown[5]= 0x08;//0x0A;
			bufDown[6]= 0x28;//val_speed_th ;
			bufDown[7]= 0x08;//0x0A;
			}
		else if(vel_th >= 0 and vel_x <=0 ){
			bufDown[4]= 0xD7;//val_speed_inv_th;
			bufDown[5]= 0xF7;
			bufDown[6]= 0xD7;//val_speed_inv_th;
			bufDown[7]= 0xF7;
	
			}
		else if(vel_th <= 0 and vel_x <=0 ){
			bufDown[4]= 0xF7;
			bufDown[5]= 0xD7;//val_speed_th;
			bufDown[6]= 0xF7;
			bufDown[7]= 0xD7;//val_speed_th;
			}		
		else
		{
			bufDown[4]= 0x08;//0x0A;
			bufDown[5]= 0x28;//val_speed_inv_th;
			bufDown[6]= 0x08;//0x0A;
			bufDown[7]= 0x28;//val_speed_inv_th;
			}
	}

	//useless code?
	//vl = left_vel;
	//vr = right_vel;


	bufDown[8]= 0x00;
	bufDown[9]= 0x00;
	bufDown[10]= 0x00;
	bufDown[11]= 0x00;
	bufDown[12]= 0x00;
	bufDown[13]= 0x00;
	bufDown[14]= 0x00;
	bufDown[15]= 0x00;
	bufDown[16]= 0x00;
	bufDown[17]= 0x00;
	bufDown[18]= 0x00;
	bufDown[19]= 0x00;
	bufDown[20]= 0x00; 
	bufDown[21]= 0x00; 
	bufDown[22]= 0x00;
	bufDown[23]= 0x00;
	bufDown[24]= 0x00;
	bufDown[25]= 0x00;

	//printf(" Ok !\n");
}

int main(int argc, char** argv){
	
	ros::init(argc, argv, "base_controller");
	ros::NodeHandle n;
	ros::Subscriber cmd_vel_sub = n.subscribe("cmd_vel", 100, cmd_velCallback);
	
	int buff_length = 10;
	
	//Defined messages that are published																							
	ros::Publisher temps_pub = n.advertise<std_msgs::Float32>("Time_stamps",buff_length);
	ros::Publisher codi_error_pub = n.advertise<std_msgs::Float32>("Error_code",buff_length);
	/**
	*Topics speeds of the wheels means FX front wheel (front)
	*Means BX rear wheel (Back) XL and XR mean wheel left and
	*right respectively
	*/	
	ros::Publisher Vel_FL_pub = n.advertise<std_msgs::Float32>("Speed_Front_Left",buff_length);
	ros::Publisher Vel_FR_pub = n.advertise<std_msgs::Float32>("Speed_Front_Right",buff_length);
	ros::Publisher Vel_BL_pub = n.advertise<std_msgs::Float32>("Speed_Back_Left",buff_length);
	ros::Publisher Vel_BR_pub = n.advertise<std_msgs::Float32>("Speed_Back_Right",buff_length);
	/** 
	* Topics of battery and VCC 
	*/
	ros::Publisher Bat_pub = n.advertise<std_msgs::Float64>("Battery_Level",buff_length);
	ros::Publisher Vcc_pub = n.advertise<std_msgs::Float64>("Battery_Voltage",buff_length);
	/** 
	* Topics intensities of the wheels work just abbreviations 
	* that slogans Speed 
	*/
	ros::Publisher Int_FL_pub = n.advertise<std_msgs::Float64>("Current_Front_Left",buff_length);
	ros::Publisher Int_FR_pub = n.advertise<std_msgs::Float64>("Current_Front_Right",buff_length);
	ros::Publisher Int_BL_pub = n.advertise<std_msgs::Float64>("Current_Back_Left",buff_length);
	ros::Publisher Int_BR_pub = n.advertise<std_msgs::Float64>("Current_Back_Right",buff_length);
	/** 
	* Topics voltage control 
	*/
	ros::Publisher Cont_FL_pub = n.advertise<std_msgs::Float32>("Control_Signal_Front_Left",buff_length);
	ros::Publisher Cont_FR_pub = n.advertise<std_msgs::Float32>("Control_Signal_Front_Right",buff_length);
	ros::Publisher Cont_BL_pub = n.advertise<std_msgs::Float32>("Control_Signal_Back_Left",buff_length);
	ros::Publisher Cont_BR_pub = n.advertise<std_msgs::Float32>("Control_Signal_Back_Right",buff_length);
	/** 
	* Topics of position 
	*/
	ros::Publisher PosX_pub = n.advertise<std_msgs::Float32>("Position_Y",buff_length);
	ros::Publisher PosY_pub = n.advertise<std_msgs::Float32>("Position_X",buff_length);
	ros::Publisher Theta_pub = n.advertise<std_msgs::Float32>("Angular_Position",buff_length);
	/** 
	* Topics of encoder
	*/
	ros::Publisher Enc_L_pub = n.advertise<std_msgs::Float32>("Encoder_Left",buff_length);
	ros::Publisher Enc_R_pub = n.advertise<std_msgs::Float32>("Encoder_Right",buff_length);
	
	/**
	 * Topic for sending the type of terrain
	 */
	ros::Publisher terrain_type_pub = n.advertise<std_msgs::Float32>("Terrain_type",buff_length);
	ros::Publisher terrain_R_pub = n.advertise<std_msgs::Int8>("Terrain_R",buff_length);
	ros::Publisher terrain_L_pub = n.advertise<std_msgs::Int8>("Terrain_L",buff_length);
	
	
	//Subscriber to the Joystick to detect when a button is pressed and check the terrain
	ros::Subscriber joy_sub = n.subscribe("joy",0,joyCallback);
	
	//ros::Subscriber test = n.subscribe("runTest", 0, runTest_Callback);
	
	ros::Rate loop_rate(20);

	while (ros::ok())
	{
		/**
		* This are a message object. You stuff it with data, and then publish it.
		*
		* We define objects message
		*/
		std_msgs::Float32 temps;
		std_msgs::Float32 codi_err;
		std_msgs::Float32 Vel_FL;
		std_msgs::Float32 Vel_FR;
		std_msgs::Float32 Vel_BL;
		std_msgs::Float32 Vel_BR;

		std_msgs::Float64 Int_FL;
		std_msgs::Float64 Int_FR;
		std_msgs::Float64 Int_BL;
		std_msgs::Float64 Int_BR;

		std_msgs::Float64 Bat;
		std_msgs::Float64 Vcc;  

		std_msgs::Float32 Cont_FL;
		std_msgs::Float32 Cont_FR;
		std_msgs::Float32 Cont_BL;
		std_msgs::Float32 Cont_BR;

		std_msgs::Float32 Pos_X;
		std_msgs::Float32 Pos_Y;
		std_msgs::Float32 Theta;
    
		std_msgs::Float32 Enc_L;
		std_msgs::Float32 Enc_R;
		
		std_msgs::Float32 Terr_type;
		std_msgs::Int8 Terr_R;
		std_msgs::Int8 Terr_L;
		
		
		double dxy = 0.0;
		double dth = 0.0;
		ros::Time current_time = ros::Time::now();
		double dt;
		double velxy = dxy / dt;
		double velth = dth / dt;
		//ros::spinOnce();
		dt = (current_time - last_time).toSec();;
		last_time = current_time;
		// calculate odomety
		if(right_enc == 0.0){
			distance_left = 0.0;
			distance_right = 0.0;
		}else{
			distance_left = (left_enc - left_enc_old) / ticks_per_meter;
			distance_right = (right_enc - right_enc_old) / ticks_per_meter;
		}
		left_enc_old = left_enc;
		right_enc_old = right_enc;
		dxy = (distance_left + distance_right) / 2.0;
		dth = (distance_right - distance_left) / width_robot;
		if(dxy != 0){
			x += dxy * cosf(dth);
			y += dxy * sinf(dth);
		}
		if(dth != 0){
			th += dth;
		}
		odom_quat = tf::createQuaternionMsgFromRollPitchYaw(0,0,th);
		loop_rate.sleep();


	struct itimerval val;			// Variable per la configuracion del timer
	struct sigaction interrupcio;		// Variable per la configuracion d'interrupcions
	struct timeval timeoutSock;		//Variable per controlar el temps d'espera dels sockets.

	//0x88 = We put at 1 bit more weight to indicate that a package with Yaw,
	//and the other one indicates that correction of degree
	
	PIC_sendBuf_minim[0] = 0x88; //0x88 = 10001000

	if ((1)){
		addrPIC = "192.168.1.18"; //BB3
		//addrPIC = "192.168.1.15"; //SMALLBOT 1
		//addrPIC = "192.168.1.13"; //BB1
				
		printf("\nX\n");
		
		printf("IP PIC: %s\n",addrPIC);

		//start the buffer down
		int k=0;
		int h=0;
		for (k=0; k<=25; k++){
			bufDown[k]=0;
		}
		for (h=0; h<=37; h++){
			bufUp[h]=0;
		}
		
	}
	else{
		printf("\n**ERROR**\n\ncommands: ./communication IP_dspic period\n\n");
		exit(0);
	}


	startSocketPIC();
	
	if(sockPIC == -1){
		printf("\nError creating socket PIC\n");
		
	}
	else{ 
		//Clean the possible data that may exist on dsPIC
		timeoutSock.tv_sec=0;
		timeoutSock.tv_usec=750000;

		setsockopt(sockPIC, SOL_SOCKET, SO_RCVTIMEO, &timeoutSock, sizeof(timeoutSock));

		printf("\nCleaning dsPIC socket...");
		
		while(recv(sockPIC, &bufUp[0], 1, 0)>=0){
			printf("receiving %d\n", bufUp[0]);	
			printf("receiving %d\n", bufUp[0]);	
		}
		printf(" Ok !\n");
	}

	// Creating THREADS
	
	if (pthread_create (&sendPICTh, NULL, sendPIC, (void *) 1) != 0){
		printf("\nError creating the thread of sendPIC\n");
	}
	if (pthread_create (&recvPICTh, NULL, recvPIC, (void *) 1) != 0)
	{
		printf("\nError creating the thread of recvPIC\n");
	}
	

	pthread_mutex_init(&mutex_nBufEnviar, NULL); //Initialize Mutex - http://en.wikipedia.org/wiki/Mutual_exclusion

	val.it_interval.tv_sec=0;
	val.it_interval.tv_usec=15000;
	
	val.it_value.tv_sec=0;
	val.it_value.tv_usec=15000;
	
	setitimer(ITIMER_REAL, &val, NULL);
	//printf ("period=%d\n",PC_send_period);

	//Interrupcion timer
	interrupcio.sa_handler = gestio_int;		// El manejador de de les senyals capturades es gestio_int
												// The handler signals are captured gestio_int
	sigemptyset (& (interrupcio . sa_mask));	// Especifiquem que no es bloquejar� cap senyal
												// Specific signal is not blocked
	interrupcio.sa_flags = 0;					// Tots els flags desactivats
												// All flags turned off
	sigaction(SIGALRM, &interrupcio, NULL);
	sigaction(SIGINT, &interrupcio, NULL);


	while(1){
		
		//Here ends the message example
		//Assign variables to messages
		
		temps.data= Vtemsp;
		codi_err.data = Vcodi_err;
		
		Bat.data=dVBat;
		Vcc.data=dVVcc;
		Vel_FL.data=VVel_FL;
		Vel_FR.data=VVel_FR;
		Vel_BL.data=VVel_BL;
		Vel_BR.data=VVel_BR;

		Int_FL.data=dVInt_FL;
		Int_FR.data=dVInt_FR;
		Int_BL.data=dVInt_BL;
		Int_BR.data=dVInt_BR;

		Cont_FL.data=Scont_FL;
		Cont_FR.data=Scont_FR;
		Cont_BL.data=Scont_BL;
		Cont_BR.data=Scont_BR;

		Pos_X.data=dPos_X;
		Pos_Y.data=dPos_Y;
		Theta.data=dTheta;
		
		Enc_L.data=dEnc_L;
		Enc_R.data=dEnc_R;
		
		Terr_type.data = terrtype;
		Terr_R.data = terrainR;
		Terr_L.data = terrainL;
		
		temps_pub.publish(temps); 
		codi_error_pub.publish(codi_err); 
		Vel_FL_pub.publish(Vel_FL); 
		Vel_FR_pub.publish(Vel_FR); 
		Vel_BL_pub.publish(Vel_BL); 
		Vel_BR_pub.publish(Vel_BR); 
		Int_FL_pub.publish(Int_FL); 
		Int_FR_pub.publish(Int_FR); 
		Int_BL_pub.publish(Int_BL); 
		Int_BR_pub.publish(Int_BR); 
		Bat_pub.publish(Bat); 
		Vcc_pub.publish(Vcc); 
		Cont_FL_pub.publish(Cont_FL); 
		Cont_FR_pub.publish(Cont_FR); 
		Cont_BL_pub.publish(Cont_BL); 
		Cont_BR_pub.publish(Cont_BR);
		PosX_pub.publish(Pos_X); 
		PosY_pub.publish(Pos_Y); 
		Theta_pub.publish(Theta);
		Enc_L_pub.publish(Enc_L);
		Enc_R_pub.publish(Enc_R);
		terrain_type_pub.publish(Terr_type);
		if (terreny == 1){
			terrain_R_pub.publish(Terr_R);
			terrain_L_pub.publish(Terr_L);
			terreny = 0;
		}
				
		send_PIC=1;
		ros::spinOnce();
		loop_rate.sleep();
		}
	return 0;
			
	}
}

void * recvPIC (void * num)
{
	//We define the variable that will be moving the pointer in different positions bufUp []
	unsigned char aux1;
	unsigned char aux2;
	unsigned char aux3;
	unsigned char aux4;
	unsigned char aux5;
	unsigned char buffer[1];
	
	while (1)
	{
		//We get the package that fixed!
		recv(sockPIC, &bufUp[0], 5, MSG_WAITALL);
		aux1 = bufUp[0];
		aux2 = bufUp[1];

		if((aux1 == 0xFF) && (aux2 == 0x00)) //to check package from BIGBOT
		{
			aux1 = bufUp[0];
			//printf("Header: %x\n",aux1);
			
			aux2 = bufUp[1];
			
			aux3 = bufUp[2];
			Vtemsp = (double)aux3;
			//rintf("\n\nTime Stamp :%d",aux3);
			
			aux4 = bufUp[3];
			Vcodi_err = (double)aux4;
			//printf("\n\nError code :%d",aux4);
				
			aux5 = bufUp[4];

			
			//Get step by step the variable
			if(fVelocitat)
			{
				recv(sockPIC, &bufUp[5], 4, MSG_WAITALL);
				aux1 = bufUp[5];
				aux2 = bufUp[6];
				aux3 = bufUp[7];
				aux4 = bufUp[8];

				VVel_FL= (int)aux1;
				if(VVel_FL > 125)
				{
					VVel_FL = VVel_FL - 257 ;
				}
				VVel_FR= (int)aux2;
				if(VVel_FR > 125)
				{
					VVel_FR = VVel_FR - 257 ;
				}
				VVel_BL= (int)aux3;
				if(VVel_BL > 125)
				{
					VVel_BL = VVel_BL - 257 ;
				}
				VVel_BR= (int)aux4;
				if(VVel_BR > 125)
				{
					VVel_BR = VVel_BR - 257 ;
				}

				//printf("\n\ntesting :%d, %d, %d, %d\n",aux1,aux2,aux3,aux4);
				//printf("\n\nVelocitats: %d, %d, %d, %d\n",VVel_FL,VVel_FR,VVel_BL,VVel_BR); // not works
				//printf("ADEU2%d, %x, %s, %s\n",veloccitatFL,velocitatFR,velocitatRL,velocitatRR); //some variable (null)
			}
			
			if(fBattery) 	
			{
				recv(sockPIC, &bufUp[9], 2, MSG_WAITALL);
				aux1 = bufUp[9];
				aux2 = bufUp[10];

				if(aux1!=0)
				{dVBat=(double)aux1/10;}
				if(aux2!=0)
				{dVVcc=((double)aux2/100)+3.5;}
				//printf("Bat: %f, %f,\n",dVBat,dVVcc); //battery and Vcc
				//printf("Bat: %i, %i,\n",aux1,aux2);  //battery and Vcc
			}
			
			
			if(fPosition) 	
			{	
				recv(sockPIC, &bufUp[11], 5, MSG_WAITALL);
				
				aux1 = bufUp[13];
				aux1 = aux1 < 8;
				aux2 = bufUp[11];
				VPos_X = aux1|aux2;
				dPos_X= (double)VPos_X/100;

				aux1 = bufUp[14];
				aux1 = aux1 < 8;
				aux2 = bufUp[12];
				VPos_Y = aux1|aux2;
				dPos_Y= (double)VPos_Y/100;

				aux1 = bufUp[15];
				dTheta = (aux1*360)/255;

				//printf("Pos: %f, %f, %d, %d,%d\n",bufUp[11],bufUp[12],bufUp[13],bufUp[14],bufUp[15]); //position x and y
				}
			
			if(fCurrent) 	
			{	
				//currents sent from the PIC are divided into 2 bytes. The values go from 0 to 2¹⁶.
				//That's why it is needed to move the first byte of each current so we can get the right value.
				//From 0 to 32768 are positive currents, from 32768 to 65536 are negative.
				recv(sockPIC, &bufUp[16], 8, MSG_WAITALL);

				dVInt_FL = (bufUp[16] << 8) | bufUp[17];

				dVInt_FR = (bufUp[18] << 8) | bufUp[19];
				
				dVInt_BL = (bufUp[20] << 8) | bufUp[21];
				
				dVInt_BR = (bufUp[22] << 8) | bufUp[23];
				
								
				//if the current is negative it is needed to substract 32768 in order to see the currents in negative because the 
				//robot is going backwards
				if (dVInt_FL>32768){
					dVInt_FL = 32768-dVInt_FL;
				}
				if (dVInt_FR>32768){
					dVInt_FR = 32768-dVInt_FR;
				}
				if (dVInt_BL>32768){
					dVInt_BL = 32768-dVInt_BL;
				}
				if (dVInt_BR>32768){
					dVInt_BR = 32768-dVInt_BR;
				}
				
				//cout << dVInt_FL << "   " << dVInt_FR << "   " << dVInt_BL << "   " << dVInt_BR << endl;
			}
			
			if(fCtlSig) 	
			{	
				recv(sockPIC, &bufUp[24], 4, MSG_WAITALL);

				aux1 = bufUp[24];
				aux2 = bufUp[25];
				aux3 = bufUp[26];
				aux4 = bufUp[27];

				Scont_FL= (int)aux1;
				Scont_FR= (int)aux2;
				Scont_BL= (int)aux3;
				Scont_BR= (int)aux4;

				//printf("Senyals de Control: %d, %d, %d, %d\n",Scont_FL, Scont_FR, Scont_BL, Scont_BR);
			}
			
			if(fDiffTick)
			{
				recv(sockPIC, &bufUp[28], 2, MSG_WAITALL);
				aux1 = bufUp[28];
				aux2 = bufUp[29];
				
				//printf("Diff Tick: %x, %x\n",aux1, aux2);
			}
			
			if(fSensors)	
			{
				recv(sockPIC, &bufUp[30], 4, MSG_WAITALL);
				aux1 = bufUp[30];
				aux2 = bufUp[31];
				aux3 = bufUp[32];
				aux4 = bufUp[33];
				
				//printf("Sensors: %x, %x, %x, %x\n",aux1, aux2, aux3, aux4);
			}
			
			
			if(fEncoders) 	
			{
				recv(sockPIC, &bufUp[34], 4, MSG_WAITALL);

				aux1 = bufUp[34];
				aux2 = bufUp[35];
				aux3 = bufUp[36];
				aux4 = bufUp[37];
				
				buffer[0] = bufUp[35] + bufUp[34];
				buffer[1] = bufUp[36] + bufUp[37];
				
				VEnc_L = (double)buffer[0];
				VEnc_R = (double)buffer[1];
				
				dEnc_L = VEnc_L;
				dEnc_R = VEnc_R;
				//printf("Encoders: %f,%f \n",dEnc_L, dEnc_R);

			}
		}
	}
}


void * sendPIC (void * num){
	//while(!start_send);
	while(1){
		sendTimer = 1;
		
		//ros::Rate loop_rate(10);
		
		//cout << "Test Speed: " << testSpeed << endl;
		//testSpeed++;
		
		if(sendTimer == 1){
			if (send_PIC==1){
				

				if (yaw_to_PIC){
					Flags_PCtoPIC = Flags_PCtoPIC|0x08;
					h_corr_t = h_yaw;
					l_corr_t = l_yaw;
				}
				
				//PIC to send the fixed part of the package
				send(sockPIC, &bufDown[0], 8, 0);

				//Send step by step the variable
				if (fPosOrder){	
					send(sockPIC, &bufDown[8],4,0);
					printf("Jo estic enviant!\n%d\n",fPosOrder);
				}
				if (fTOrder)	send(sockPIC, &bufDown[12],1,0);
				if (fCorrPos)	send(sockPIC, &bufDown[13],4,0);
				if (fCorrT)	send(sockPIC, &bufDown[17],2,0);
				if (fErrAd)	send(sockPIC, &bufDown[19],1,0);
				if (fConfPID)	send(sockPIC, &bufDown[20],6,0);
				
				//printf("Jo estic enviant!\n%d\n",fPosOrder);
				//printf("I am sending: %x, %x, %x, %x\n",bufDown[8], bufDown[12], bufDown[13], bufDown[17]);

				send_PIC=0;
			}

			else if(yaw_to_PIC == 1){
				//Enviem la capçalera del paquet "mini"
				send(sockPIC, &PIC_sendBuf_minim[0], 1, 0);
				h_corr_t = h_yaw;
				l_corr_t = l_yaw;
				send(sockPIC, &bufDown[17], 2, 0);
			}

			sendTimer = 0;
		}
		
		//ros::spinOnce();
		//loop_rate.sleep();
	}

}


void startSocketPIC(){
	sockPIC = CreateTCPClient(addrPIC,10001);
printf("\n entra al startsocketPIC %d %s", sockPIC, addrPIC );
}

void stopSocketPIC(){
	CloseSock(sockPIC);
}

// Gestio de les senyals capturades
//Manage signals captured
void gestio_int (int numero)
{
	switch (numero) 
	{
	case SIGALRM :
		//Flag que controla quan enviar al PIC per conservar l'ample de banda
		sendTimer = 1;

		break;

	case SIGINT:
		/**********************************************
		Codi executat per la interrupció de Ctrl+C. A part, està configurat per que quan Labview es desconecti
		salti aquesta interrupció.

		Aquesta part ens assegura que la transmissió de l'IMU es "tanca" correctament i que no queda res
		al buffer.
		***********************************************/
		printf("\nS'ha pulsat Ctrl+C!\n");
		

		printf("\nFinalitzant comunicacio amb el PIC...");
		
		CloseSock(sockPIC);

		printf("\nSortint ...\n\n");
		exit(1);
		break;
	}
}


//FUNCIONS DE TCPSOCK


int AcceptConn(int Sock,char * RemoteAddr)
{
	struct sockaddr_in addr;
	int addrSize = sizeof(struct sockaddr_in);
	int s;

	s = accept(Sock,(struct sockaddr *)&addr,(socklen_t *)&addrSize);

	strcpy(RemoteAddr,inet_ntoa(addr.sin_addr));

	return s;
}

int CreateTCPClient(char * HostName,int Port)
{
	cout << "Versió 1.1" << endl;
	printf("\n entra al create tcpclient");
	int s;
	struct sockaddr_in dest_addr;

	s = CreateSock(SOCK_STREAM);

	if ( s == -1 )
		return -1;

	dest_addr.sin_family = AF_INET;
	dest_addr.sin_port = htons(Port);
	HostNameToAddr(HostName,&dest_addr);

   if ( connect(s, (struct sockaddr *)&dest_addr, sizeof(struct sockaddr)) == -1 )
	{ printf("\n PETA AQUI MERDA! %d \n", errno);
			return -1;
	}
    return s;
}

int TCPSend(int Sock,char * Data,int Len)
{
	return send(Sock, Data, Len, 0);
}

int TCPRecv(int Sock,char * Buf,int Len)
{

	return recv(Sock, Buf, Len, 0);
}

		//FUNCIONS DE SOCKBASE

int CreateSock(int SockType)
{
printf("\n entra al createsock" );
	return socket(AF_INET,SockType,0);

}


int HostNameToAddr(char * HostName,struct sockaddr_in * Addr)
{
	struct hostent *hp;

  	hp=gethostbyname(HostName);
  	if(hp==(struct hostent *)0)
  		return -1;

  	memcpy((char *) &Addr->sin_addr,(char *)hp->h_addr,hp->h_length);


  	return 0;
}


void CloseSock(int Sock)
{
	if ( close(Sock) == -1 )
		perror("CloseSock");
}
